	org 0x7c00
	bits 16
	cpu 486

	STAGE2 equ 0x7e00

	mov esp, 0x7c00				;top of stack
	mov ebp, 0					;bottom of stack

	mov ax, 0
	mov ds, ax
	mov ss, ax
	mov es, ax
	mov fs, ax
	mov gs, ax

	;; Init screen
	mov ah, 0x0
	mov al, 0x03
	int 0x10

	;; Write string
	mov bp, STR_DISK_START
	mov cx, 41
	call WRITE_STRING

	;; Read in OS
	mov ch, 0					;cylinder
	;; sector 1 is bootloader
	;; so we start at sector 2
	mov cl, 2					;sector
	mov dh, 0					;head
	push word 0x7e0				;0x7e00 / 0x10
	pop es
READ_NEXT:
	mov ah, 0x02				;Read
	mov al, 1					;Read 1 sector
	;; bios puts boot drive in dl
	;mov dl, 0x0					;drive #0 (A:)
	mov bx, 0					;segment:offset, offset = 0
	int 0x13
	jc READ_ERROR

	cmp al, 1
	jne READ_ERROR
	
	;; reset read attempt counter
	mov ah, 0
	mov [READ_ATTEMPTS], ah

	mov ax, es
	call WRITE_HEX
	mov ax, [es: bx]
	call WRITE_HEX
	mov ax, dx
	call WRITE_HEX
	
	;; inc es
	mov ax, es
	add ax, 512 / 16
	mov es, ax
	
	;; inc sector count
	inc cl
	cmp cl, 19					;18 sectors
	je INC_HEAD
	
	jmp READ_NEXT

INC_HEAD:
	mov cl, 1					;reset sector count
	inc dh
	cmp dh, 2
	je INC_CYL
	jmp READ_NEXT
	
INC_CYL:
	mov dh, 0					;reset head count
	inc ch						;inc cyl
	cmp ch, 5					;read 19 cylinders
	;; 19 * 18 * 2 * 512 = 350kb
	je READ_DONE
	jmp READ_NEXT
	
READ_ERROR:
	mov ah, [READ_ATTEMPTS]
	inc ah
	cmp ah, 5
	jge READ_GIVEUP
	mov [READ_ATTEMPTS], ah
	jmp READ_NEXT

READ_GIVEUP:
	;; Print err message
	mov bp, STR_DISK_FAIL
	mov cx, 41
	call WRITE_STRING
	
	jmp $

READ_DONE:
	;; Write string
	mov bp, STR_DISK_DONE
	mov cx, 41
	call WRITE_STRING
	push word 0
	pop es
	
	jmp STAGE2
	
	;; Put string in bp
	;; put length in cx
WRITE_STRING:
	pusha
	mov ah, 0x13
	mov al, 0x01
	mov bh, 0
	mov bl, 7
	push word 0
	pop es
	mov dl, 0
	mov dh, 0
	int 0x10
	popa
	ret

	;; input in eax
WRITE_HEX:
	push eax					;need to be explicit about 32 bit registers
	pusha
	push ax
	mov ah, 0x0e
	mov al, '0'
	mov bx, 0
	int 0x10
	mov ah, 0x0e
	mov al, 'x'
	mov bx, 0
	int 0x10
	pop ax
	mov dx, 0
WH_LOOP:
	rol eax, 4
	inc dx
	cmp dx, 8
	jg WH_DONE
	mov cx, ax
	and cl, 0xF
	cmp cl, 9
	jg WH_HEX
	add cl, 0x30				;dec -> ascii
	push ax
	mov ah, 0x0e
	mov al, cl
	mov bx, 0
	int 0x10
	pop ax
	jmp WH_LOOP

WH_HEX:
	add cl, 0x37
	push ax
	mov ah, 0x0e
	mov al, cl
	mov bx, 0
	int 0x10
	pop ax
	jmp WH_LOOP

WH_DONE:
	mov ah, 0x0e
	mov al, ' '
	mov bx, 0
	int 0x10
	mov al, ' '
	int 0x10
	popa
	pop eax
	ret
	
READ_ATTEMPTS:	db 0
STR_DISK_START:	db "Loading SPEEDOS from floppy diskette...", 0xA, 0xD, 0
STR_DISK_FAIL:	db "Error! Disk seems to be corrupted!     ", 0xA, 0xD, 0
STR_DISK_DONE:	db "Ready to load second stage!            ", 0xA, 0xD, 0
	
	times 510-($-$$) db 0
	dw 0xaa55
